Scanner = class()
Scanner.vertexSet = nil
Scanner.edgeSet = nil
Scanner.colourMap = nil
Scanner.startVertices = nil
Scanner.scanned = nil

function Scanner:init(startVertices)
    self.vertexSet = VertexSet()
    self.edgeSet = EdgeSet()
    self.colourMap = Dictionary()
    self.startVertices = startVertices
    self.scanned = false
end

local function getBooleanOperation(interactable)
    local booleanOperation = interactable:getUvFrameIndex() % 6 + 1

    if #interactable:getParents() == 1 then
        if booleanOperation < 4 then
            booleanOperation = BooleanOperationEnum.Identity
        else
            booleanOperation = BooleanOperationEnum.Negation
        end
    end

    return booleanOperation
end

local function scanRecursive(self, interactable)
    local shapeUuid = tostring(interactable:getShape():getShapeUuid())


    if not (shapeUuid == "9f0f56e8-2c31-4d83-996c-d00a9b296c3f"
    or shapeUuid == "8f98db04-72eb-4a3a-88a1-f4f3e8d818ee") then
        error("Unsupported interactable found")
    end

    if self.vertexSet:getVertexById(interactable:getId()) == nil then
        local outputVertex = Gate(interactable:getId(), getBooleanOperation(interactable))
        self.vertexSet:add(outputVertex)
        self.colourMap:add(outputVertex:getIdentifier(), tostring(interactable:getShape():getColor()))

        for _, child in ipairs(interactable:getChildren()) do
            scanRecursive(self, child)
            local inputVertex = self.vertexSet:getVertexById(child:getId())
            self.edgeSet:add(Edge(outputVertex, inputVertex))
        end
    end
end

function Scanner:scan()
    for _, interactable in self.startVertices:getIterator() do
        scanRecursive(self, interactable)
    end

    self.scanned = true
end

function Scanner:getVertexSet()
    if not self.scanned then
        error("You need to call scan() at least once in order to obtain a vertex set")
    end

    return self.vertexSet
end

function Scanner:getEdgeSet()
    if not self.scanned then
        error("You need to call scan() at least once in order to obtain an edge set")
    end

    return self.edgeSet
end

function Scanner:getColourMap()
    if not self.scanned then
        error("You need to call scan() at least once in order to obtain a colour lookup table")
    end

    return self.colourMap
end
