CircuitIO = class()
CircuitIO.availablePins = nil
CircuitIO.pinColourMap = nil
CircuitIO.connectedPins = nil
CircuitIO.sequence = nil
CircuitIO.mode = nil

function CircuitIO:init(pins, colourMap, mode)
    self.availablePins = Dictionary()
    self.pinColourMap = Dictionary()
    self.connectedPins = Dictionary()
    self.sequence = Dictionary()
    self.mode = mode

    for _, pin in pins:getIterator() do
        self.availablePins:add(pin, pin)
        self.pinColourMap:add(pin, colourMap:get(pin))
        self.sequence:add(pin, false)
    end
end

function CircuitIO:attach(ioShape, targetPin)
    local containsKey, pin = self.availablePins:tryGet(targetPin)

    if containsKey then
        self.availablePins:remove(pin)
        self.connectedPins:add(pin, ioShape)
    else
        error("Pin not available")
    end
end

function CircuitIO:detach(ioShape, targetPin)
    local connectedShape = self.connectedPins:get(targetPin)

    if ioShape == connectedShape then
        self.connectedPins:remove(targetPin)
        self.availablePins:add(targetPin, targetPin)
    else
        error("Shape was not connected to the target pin")
    end
end

function CircuitIO:getSequence()
    local value, containsKey, ioShape

    for pin, _ in self.sequence:getIterator() do
        value = false
        containsKey, ioShape = self.connectedPins:tryGet(pin)

        if containsKey then
            value = ioShape:server_isActive()
        end

        self.sequence:update(pin, value)
    end

    return self.sequence
end

function CircuitIO:setSequence(sequence)
    if self.mode == IOModeEnum.Output then
        local containsKey, ioShape

        for pin, value in sequence:getIterator() do
            self.sequence:update(pin, value)
            containsKey, ioShape = self.connectedPins:tryGet(pin)

            if containsKey then
                ioShape:server_setActive(value)
            end
        end
    else
        error("Can't set sequence when in input mode")
    end
end

function CircuitIO:getAvailablePin(targetColour)
    if not self.availablePins:isEmpty() then
        local availablePin

        for pin, _ in self.availablePins:getIterator() do
            availablePin = pin
        end

        for pin, colour in self.pinColourMap:getIterator() do
            if targetColour == colour and self.availablePins:containsKey(pin) then
                availablePin = pin
                break
            end
        end

        return availablePin
    else
        error("No pins available")
    end
end
