dofile("../Include.lua")

Input = class(ScriptedShape)
Input.maxParentCount = 1
Input.maxChildCount = 1
Input.connectionInput =  sm.interactable.connectionType.logic
Input.connectionOutput = sm.interactable.connectionType.logic
Input.colorNormal = sm.color.new(0x349E1AFF)
Input.colorHighlight = sm.color.new(0x7EEB63FF)
Input.circuit = nil
Input.pin = nil
Input.nameTag = nil
Input.initWithStorage = nil

local function onChildConnectedEvent(sender, eventArgs)
    local shape = eventArgs.connections[1]:getShape()

    if tostring(shape:getShapeUuid()) == "6b8547b9-407b-40a4-a978-745713b581b9" then
        sender.circuit = sm.game.scriptedShapes:getShapeByShapeId(shape:getId())
        sender.circuit:server_addInput(sender, tostring(sender.shape:getColor()), sender.pin)
        sender.storage:save({ pin = sender.pin })
    end
end

local function onChildDisconnectedEvent(sender, eventArgs)
    if sender.circuit ~= nil then
        sender.circuit:server_removeInput(sender, sender.pin)
        sender.circuit = nil
        sender.storage:save({ pin = sender.pin })
    end
end

local function onColourChangedEvent(sender, eventArgs)
    if sender.circuit ~= nil then
        sender.circuit:server_removeInput(sender, sender.pin)
        sender.circuit:server_addInput(sender, tostring(eventArgs.colour), sender.pin)
        sender.storage:save({ pin =  sender.pin })
    end
end

function Input:init()
    ScriptedShape.init(self)
    _ = self.events:get("childConnectedEvent") + onChildConnectedEvent
    _ = self.events:get("childDisconnectedEvent") + onChildDisconnectedEvent
    _ = self.events:get("colourChangedEvent") + onColourChangedEvent
end

function Input:server_onCreate()
    ScriptedShape.server_onCreate(self)
    self.initWithStorage = self:server_hasStoredData()
end

function Input:server_onDestroy()
    ScriptedShape.server_onDestroy(self)

    if self.circuit ~= nil then
        self.events:get("childDisconnectedEvent")(self, { connectionCount = 0, connections = {} })
    end
end

function Input:server_onFixedUpdate(deltaTime)
    ScriptedShape.server_onFixedUpdate(self, deltaTime)

    if self.initWithStorage then
        self.pin = self.storage:load().pin
        self.events:get("childConnectedEvent")(self, { connectionCount = #self.interactable:getChildren(), connections = self.interactable:getChildren() })
        self.initWithStorage = false
    end

    local parent = self.interactable:getSingleParent()
    self.interactable:setActive(parent ~= nil and parent:isActive() or false)
end

function Input:server_isActive()
    local parent = self.interactable:getSingleParent()

    return parent ~= nil and parent:isActive() or false
end

function Input:server_setPin(pin)
    if sm.exists(self.shape) then
        self.pin = pin
        self.network:sendToClients("client_setNameTagText", pin)
    end
end

function Input:server_resetConnectionState()
    self.circuit = nil
    self.pin = nil
    self.storage:save({})
end

function Input:client_onCreate()
    self.nameTag = sm.gui.createNameTagGui()
    self.nameTag:setWorldPosition(self.shape:getWorldPosition() + sm.vec3.new(0, 0, 0.25))
    self.nameTag:setRequireLineOfSight(false)
    self.nameTag:setMaxRenderDistance(10)
end

function Input:client_onDestroy()
    self.nameTag:close()
end

function Input:client_onUpdate(deltaTime)
    self.interactable:setUvFrameIndex(self.interactable:isActive() and 6 or 0)

    if _G.gtEnabled then
        self.nameTag:setWorldPosition(self.shape:getWorldPosition() + sm.vec3.new(0, 0, 0.25))
        self.nameTag:open()
    else
        self.nameTag:close()
    end
end

function Input:client_setNameTagText(text)
    local formattedText = string.format("#7EEB63%s", text or "")
    self.nameTag:setText("Text", formattedText)
end
