dofile("../Include.lua")

InstantLogicGate = class(ScriptedShape)
InstantLogicGate.maxParentCount = -1
InstantLogicGate.maxChildCount = -1
InstantLogicGate.connectionInput = sm.interactable.connectionType.logic
InstantLogicGate.connectionOutput = sm.interactable.connectionType.logic
InstantLogicGate.colorNormal = sm.color.new(0xa35c00ff)
InstantLogicGate.colorHighlight = sm.color.new(0xd1853eff)
InstantLogicGate.lastExecutedTick = nil
InstantLogicGate.operation = nil
InstantLogicGate.gui = nil
InstantLogicGate.currentState = nil
InstantLogicGate.delayOutput = nil
InstantLogicGate.guiOperationLookup = {
    And = { numericId = 0, descriptionText = "#{CONTROLLER_LOGIC_DESCRIPTION_AND}"},
    Or = { numericId = 1, descriptionText = "#{CONTROLLER_LOGIC_DESCRIPTION_OR}"},
    Xor = { numericId = 2, descriptionText = "#{CONTROLLER_LOGIC_DESCRIPTION_XOR}"},
    Nand = { numericId = 3, descriptionText = "#{CONTROLLER_LOGIC_DESCRIPTION_NAND}"},
    Nor = { numericId = 4, descriptionText = "#{CONTROLLER_LOGIC_DESCRIPTION_NOR}"},
    Xnor = { numericId = 5, descriptionText = "#{CONTROLLER_LOGIC_DESCRIPTION_XNOR}"},
}

function InstantLogicGate:init()
    ScriptedShape.init(self)
    self.lastExecutedTick = 0
    self.currentState = false
    self.delayOutput = false
end

function InstantLogicGate:server_onCreate()
    ScriptedShape.server_onCreate(self)

    if ScriptedShape.server_hasStoredData(self) then
        self.operation = self.storage:load().operation
    else
        self.operation = 0
    end

    self.network:setClientData({ operation = self.operation })
    self.storage:save({ operation = self.operation })
end

function InstantLogicGate:server_onFixedUpdate(deltaTime)
    ScriptedShape.server_onFixedUpdate(self, deltaTime)
    self:server_computeRecursive()
end

function InstantLogicGate:server_computeRecursive()
    if self.lastExecutedTick >= sm.game.getCurrentTick() then
        return
    end

    self.lastExecutedTick = sm.game.getCurrentTick()
    local operationFunction = InstantBooleanOperation[(self.operation % 3) + 1]
    -- Set runningState to identity element, depending on operation
    local runningState = (self.operation % 3)  == 0
    local breakCondition

    if #self.interactable:getParents() > 0 then
        for _, interactable in ipairs(self.interactable:getParents()) do
            if tostring(interactable:getShape():getShapeUuid()) == "8f98db04-72eb-4a3a-88a1-f4f3e8d818ee" and not self.delayOutput then
                local scriptedShape = sm.game.scriptedShapes:getShapeByInteractableId(interactable:getId())
                scriptedShape:server_computeRecursive()
                runningState, breakCondition = operationFunction(runningState, scriptedShape:server_getState())
            else
                runningState, breakCondition = operationFunction(runningState, interactable:isActive())
            end

            if breakCondition then
                break
            end
        end

        if self.operation < 3 then
            self.currentState = runningState
        else
            self.currentState = not runningState
        end
    else
        self.currentState = false
    end

    self.interactable:setActive(self.currentState or false)
end

function InstantLogicGate:server_onDestroy()
    ScriptedShape.server_onDestroy(self)
end

function InstantLogicGate:server_changeOperation(operation)
    self.operation = operation
    self.storage:save({ operation = operation })
    self.network:setClientData({ operation = operation })
end

function InstantLogicGate:server_getState()
    return self.currentState
end

function InstantLogicGate:client_onUpdate(deltaTime)
    self.interactable:setUvFrameIndex(self.interactable.active and self.operation + 6 or self.operation)
end

function InstantLogicGate:client_onDestroy()
    if self.gui ~= nil then
        self.gui:close()
    end
end

function InstantLogicGate:client_onClientDataUpdate(data)
    self.operation = data.operation
end

function InstantLogicGate:client_onInteract(character, lookingAt)
    if lookingAt then
        if self.gui == nil then
            self:gui_initialise()
        end

        self.gui:open()
    end
end

function InstantLogicGate:gui_initialise()
    self.gui = sm.gui.createGuiFromLayout("$MOD_DATA/Gui/Interactable_LogicGate.layout")

    for k, operation in pairs(self.guiOperationLookup) do
        self.gui:setButtonCallback(k, "gui_setOperation")
        self.gui:setButtonState(k, operation.numericId == self.operation)
    end

    self:gui_updateLamps(self.operation)

    self.gui:setOnCloseCallback("gui_onCloseCallback")
end

function InstantLogicGate:gui_setOperation(buttonName)
    self:gui_disableAllButtons()
    self.network:sendToServer("server_changeOperation", self.guiOperationLookup[buttonName].numericId)
    self.gui:setButtonState(buttonName, true)
    self.gui:setText("DescriptionText", self.guiOperationLookup[buttonName].descriptionText)
    self:gui_updateLamps(self.guiOperationLookup[buttonName].numericId)
end

function InstantLogicGate:gui_disableAllButtons()
    for k, v in pairs(self.guiOperationLookup) do
        self.gui:setButtonState(k, false)
    end
end

function InstantLogicGate:gui_updateLamps(operation)
    local operationFunction = InstantBooleanOperation[(operation % 3) + 1]
    local l00 = operationFunction(false, false)
    local l01 = operationFunction(false, true)
    local l11 = operationFunction(true, true)

    if operation > 2 then
        l00 = not l00
        l01 = not l01
        l11 = not l11
    end

    self.gui:setVisible("Lamp00_off", not l00)
    self.gui:setVisible("Lamp01_off", not l01)
    self.gui:setVisible("Lamp11_off", not l11)
    self.gui:setVisible("Lamp00_on", l00)
    self.gui:setVisible("Lamp01_on", l01)
    self.gui:setVisible("Lamp11_on", l11)
end

function InstantLogicGate:gui_onCloseCallback()
    self.gui:close()
end
