dofile("../Include.lua")

Output = class(ScriptedShape)
Output.maxParentCount = 1
Output.maxChildCount = -1
Output.connectionInput =  sm.interactable.connectionType.logic
Output.connectionOutput = sm.interactable.connectionType.logic
Output.colorNormal = sm.color.new(0x469E9DFF)
Output.colorHighlight = sm.color.new(0x2AEBE8FF)
Output.circuit = nil
Output.pin = nil
Output.nameTag = nil
Output.initWithStorage = nil

local function onParentConnectedEvent(sender, eventArgs)
    local shape = eventArgs.connections[1]:getShape()

    if tostring(shape:getShapeUuid()) == "6b8547b9-407b-40a4-a978-745713b581b9" then
        sender.circuit = sm.game.scriptedShapes:getShapeByShapeId(shape:getId())
        sender.circuit:server_addOutput(sender, tostring(sender.shape:getColor()), sender.pin)
        sender.storage:save({ pin = sender.pin })
    end
end

local function onParentDisconnectedEvent(sender, eventArgs)
    if sender.circuit ~= nil then
        sender.circuit:server_removeOutput(sender, sender.pin)
        sender.circuit = nil
        sender.storage:save({ pin = sender.pin })
    end
end

local function onColourChangedEvent(sender, eventArgs)
    if sender.circuit ~= nil then
        sender.circuit:server_removeOutput(sender, sender.pin)
        sender.circuit:server_addOutput(sender, tostring(eventArgs.colour), sender.pin)
        sender.storage:save({ pin =  sender.pin })
    end
end

function Output:init()
    ScriptedShape.init(self)
    _ = self.events:get("parentConnectedEvent") + onParentConnectedEvent
    _ = self.events:get("parentDisconnectedEvent") + onParentDisconnectedEvent
    _ = self.events:get("colourChangedEvent") + onColourChangedEvent
end

function Output:server_onCreate()
    ScriptedShape.server_onCreate(self)
    self.initWithStorage = self:server_hasStoredData()
end

function Output:server_onDestroy()
    ScriptedShape.server_onDestroy(self)

    if self.circuit ~= nil then
        self.events:get("parentDisconnectedEvent")(self, { connectionCount = 0, connections = {} })
    end
end

function Output:server_onFixedUpdate(deltaTime)
    ScriptedShape.server_onFixedUpdate(self, deltaTime)

    if self.initWithStorage then
        self.pin = self.storage:load().pin
        self.events:get("parentConnectedEvent")(self, { connectionCount = #self.interactable:getParents(), connections = self.interactable:getParents() })
        self.initWithStorage = false
    end
end

function Output:server_setActive(active)
    self.interactable:setActive(active)
end

function Output:server_setPin(pin)
    if sm.exists(self.shape) then
        self.pin = pin
        self.network:sendToClients("client_setNameTagText", pin)
    end
end

function Output:server_resetConnectionState()
    self.circuit = nil
    self.pin = nil
    self.storage:save({})
end

function Output:client_onCreate()
    self.nameTag = sm.gui.createNameTagGui()
    self.nameTag:setWorldPosition(self.shape:getWorldPosition() + sm.vec3.new(0, 0, 0.25))
    self.nameTag:setRequireLineOfSight(false)
    self.nameTag:setMaxRenderDistance(10)
end

function Output:client_onDestroy()
    self.nameTag:close()
end

function Output:client_onUpdate(deltaTime)
    self.interactable:setUvFrameIndex(self.interactable:isActive() and 6 or 0)

    if _G.gtEnabled then
        self.nameTag:setWorldPosition(self.shape:getWorldPosition() + sm.vec3.new(0, 0, 0.25))
        self.nameTag:open()
    else
        self.nameTag:close()
    end
end

function Output:client_setNameTagText(text)
    local formattedText = string.format("#2AEBE8%s", text or "")
    self.nameTag:setText("Text", formattedText)
end
